#!/bin/bash

# paths that will need to be changed:
# where this script is located 
home='/home/wlewis/NASAPROJ/REMAP'
# work directory
work='/data/users/wlewis/NASAPROJ'
# where the CMAQ (METCRO2D) files are located 
metdir='/iliad/proxy/mharkey/CMAQv5/MCIP/conus12/YNT_SST'
# where the MODIS and MODIS lat/lon files are located 
modisdir='/odyssey/proxy/bpierce/LMOS/MODIS/MCD15A2H/'
# path to NETCDF ncdump (to avoid conflict with HDF's ncdump,
# which occurs on some systems) 
ncd='/opt/netcdf3/3.6.3-intel-15.0-2/bin/ncdump'

# paths that should not need to be changed:
# where the modifed METCRO2D files go
met_outdir=${work}'/METCRO2D_OUT/'
# executables directory (where the FORTRAN executable(s) are) (DO NOT CHANGE)
bindir=${home}'/bin/'
# where the local MODIS directory will be created
# (the directory will be created automatically by 
#  the script using this path)
modislocdir=${work}'/MODISLOC/'
# temporary directory 
temp=${work}'/TEMP'

# number of tiles in the mosaic
ntiles=25

# create a list of met files
flist=`ls ${metdir} | grep "METCRO2D"`
nf=`echo ${flist} | wc -w`

# create a list of modis files
mlist=`ls ${modisdir} | grep "hdf" | grep -v "xml"`
# create a list of corresopnding lat/lon files
llist=`ls ${modisdir} | grep "latlon"`

# create local directory containing NETCDF version 
# of MODIS LAI tiles (only needs to be done once)
if [ -d "$modislocdir" ] && [ "$(ls -A "$modislocdir")" ] 
 then
    echo You have already created local copies of the MODIS tiles
    echo Proceeding with LAI transplant.
 else
  echo Local copies of MODIS NETCDF files do not exist.
  echo Creating them now
  mkdir ${modislocdir}
  for each in $mlist
    do
     ncl_convert2nc ${modisdir}'/'"$each" -v Lai_500m -o ${modislocdir}
   done
  for each in $llist
    do
     cp ${modisdir}'/'"$each" ${modislocdir}
   done
 fi

# create local directory(ies) where output files will be saved
if [ -d "$met_outdir" ]
 then
    echo You have already created the Local Output Directory 
 else
    echo Creating Local Output Directory
    mkdir ${met_outdir}
 fi

if test -d ${temp}
 then
  rm -r ${temp}
  mkdir ${temp}
 else
  mkdir ${temp}
 fi

cp ${bindir}'remap_modis.exe' ${temp}

cd ${temp}

echo ${nf} >> datefile

echo '"'$met_outdir/'"' > metpath
echo '"'$modislocdir'"' > modispath

# met files:

let n=1

 for fname in $flist
  do

#  make copy of file in output directory
#  (this is the file that will be modified)
   cp ${metdir}'/'${fname} ${met_outdir}

#  parse calendar information
   yyyymmdd=`echo ${fname} | awk -F_ '{print $2}'`
   yr=`echo ${yyyymmdd} | cut -c1-4`
   mn=`echo ${yyyymmdd} | cut -c5-6`
   dy=`echo ${yyyymmdd} | cut -c7-8`
   dtg=${yr}${mn}${dy}

   echo ${dtg} >> dtglist

   gname='GRIDCRO2D_'${dtg}'.nc'
   cp ${metdir}'/'${gname} ${met_outdir}

#  convert calendar month/day to day of year
   doy=`date -d ${yr}${mn}${dy} +%j`

   days=''
   nfiles=0

#  find MODIS files within +/- 7 days of target date
   for delta in -7 -6 -5 -4 -3 -2 -1 0 1 2 3 4 5 6 7
    do
     tempyr=`date -d ${delta}" days ${yr}"-"${mn}"-"${dy}" +"%Y"`
     tempdoy=`date -d ${delta}" days ${yr}"-"${mn}"-"${dy}" +"%j"`
     mdstr='A'${tempyr}${tempdoy}
#    search for a match
     mdlist=`ls ${modislocdir} | grep ${mdstr} | grep ".nc"`
     if test -n "${mdlist}"
      then
       days=${days}" "${tempdoy}
       let nfiles=nfiles+1
        for each in ${mdlist}
         do
#         determine corresponding latlon file and output both to file list
          tilestr=`echo "${each}" | awk -F. '{print $3}'`
          tilename=`ls ${modislocdir} | grep ${tilestr} | grep "latlon"`
          echo "${each} "${tilename} >> files.${dtg}
         done 
      fi
    done 

   echo ${doy} > info.${dtg}
   echo ${nfiles} >> info.${dtg}
   echo ${days} >> info.${dtg}
   echo ${ntiles} >> info.${dtg}
   echo ${fname} >> info.${dtg}
   echo ${gname} >> info.${dtg}
#  the x and y dimensions of the met grid
   ${ncd} -h ${metdir}'/'${gname} | grep "COL =" | awk -F= '{print $2}' | awk -F';' '{print $1}' >> info.${dtg}
   ${ncd} -h ${metdir}'/'${gname} | grep "ROW =" | awk -F= '{print $2}' | awk -F';' '{print $1}' >> info.${dtg}

  done

  ./remap_modis.exe

exit
