program remap_modis

implicit none

integer :: i, j, m, ng
integer :: n, ndtg
integer :: doy, nfiles, ntiles
integer, allocatable :: days(:)
integer, allocatable :: gridfit(:)
character*80 :: modpath, metpath
character*80 :: metpath_file, modpath_file
character*80 :: metfile, gridfile
character*80 :: mod1(100), mod2(100), latlon1(100), latlon2(100)
character*120 :: mfull,gfull,metfull,latfull
character*8 :: dates(1000)
character*6, parameter :: files='files.'
character*5, parameter :: infos='info.'
character*5, parameter :: outs='test.'
character*14 :: fname
character*13 :: iname
character*120 :: outname

integer, parameter :: nx=2400, ny=2400, nlev=40
real :: modislai1(nx,ny), modislai2(nx,ny)
real :: lat(nx,ny),lon(nx,ny)
real :: wt
real :: xmin,xmax,ymin,ymax
real :: grdspc

integer :: nxg, nyg
real, allocatable :: latg(:,:), long(:,:), lai_remap(:,:)
integer, allocatable :: lmask(:,:)
real :: xming,yming,xmaxg,ymaxg

metpath_file='metpath'
modpath_file='modispath'

open(1,file=trim(modpath_file),status='old',form='formatted')
read(1,*) modpath
close(1)

open(1,file=trim(metpath_file),status='old',form='formatted')
read(1,*) metpath
close(1)

open(1,file='datefile',status='old',form='formatted')
read(1,*) ndtg
close(1)

open(1,file='dtglist',status='old',form='formatted')
do n=1,ndtg
 read(1,*) dates(n)
enddo
close(1)

! loop over all available dates
do n=1,ndtg

 iname=infos//dates(n)
 fname=files//dates(n)
 
 print*,'Processing: ',dates(n)

 open(1,file=trim(iname),status='old',form='formatted')
 read(1,*) doy
 read(1,*) nfiles
 allocate(days(nfiles))
 read(1,*) (days(i),i=1,nfiles)
 read(1,*) ntiles
 read(1,*) metfile
 read(1,*) gridfile
 read(1,*) nxg
 read(1,*) nyg
 close(1)

 outname=trim(metpath)//trim(metfile)

 if (nfiles==1) then
  wt=1.0
 else
  wt=float(days(2)-doy)/float(days(2)-days(1))
 endif

 deallocate(days)

if (n==1) then

 allocate(latg(nxg,nyg),long(nxg,nyg),lai_remap(nxg,nyg))
 allocate(lmask(nxg,nyg))

endif

! get target grid lat and lon, extrema 
 gfull=trim(metpath)//trim(gridfile)
 call read_metgrid(nxg,nyg,latg,long,lmask,gfull)
 yming=minval(latg)
 xming=minval(long)
 ymaxg=maxval(latg)
 xmaxg=maxval(long)

!target grid spacing (lat/lon) 
 grdspc=(ymaxg-yming)/float(nyg-1)

! read in the list of modis / latlon files for time 1
 open(1,file=trim(fname),status='old',form='formatted')
 do m=1,ntiles
  read(1,*) mod1(m),latlon1(m)
 enddo

! repeat for time 2, if necessary
 if (nfiles==2) then
 do m=1,ntiles
  read(1,*) mod2(m),latlon2(m)
 enddo
 endif
 close(1)

! initalize target-grid LAI array
 lai_remap(:,:)=0.

! loop over tiles
 do m=1,ntiles

  latfull=trim(modpath)//trim(latlon1(m))
  call read_latlon(lat,lon,latfull)
  xmin=minval(lon)
  xmax=maxval(lon)
  ymin=minval(lat)
  ymax=maxval(lat)

  mfull=trim(modpath)//trim(mod1(m))
  call read_modislai(modislai1,mfull)

  if (nfiles==2)then
   mfull=trim(modpath)//trim(mod2(m))
   call read_modislai(modislai2,mfull)
  endif

! remap MODIS LAI to target grid
  call remap(nx,ny,lat,lon,modislai1,modislai2,nxg,nyg,latg, & 
             long,lmask,grdspc,lai_remap,nfiles,wt)

 enddo

! write LAI to target grid file
 call write_modislai(nxg,nyg,lai_remap,lmask,latg,long,outname)

enddo

print*,'Finished Remapping MODIS LAI'

deallocate(latg,long,lai_remap,lmask)

end program remap_modis

subroutine read_latlon(lat,lon,fname)

integer, parameter :: nx=2400, ny=2400
real :: lat(nx,ny), lon(nx,ny), xtemp
integer :: temp, start1, start2, ncnt
character*120 :: fname

start1=6
start2=start1+nx*ny

open(5,file=trim(fname),form='unformatted',access='direct',recl=1)

! longitude
do ii=1,nx*ny
 j=1+(ii-1)/nx
 i=ii-nx*(j-1)
 read(5,rec=ii+start1-1) lon(i,j)
enddo

! latitude
do ii=1,nx*ny
 j=1+(ii-1)/nx
 i=ii-nx*(j-1)
 read(5,rec=ii+start2-1) lat(i,j)
enddo

close(5)

end subroutine read_latlon

subroutine read_metgrid(nx,ny,lat,lon,lmask,fname)

use netcdf

integer :: nx, ny, ncid, latid, lonid, mid, nstatus
real :: lat(nx,ny), lon(nx,ny)
integer :: lmask(nx,ny)
character*120 :: fname

! the lat and lon in this file have two leading dimensions 
! of length 1, may have to account for that

nstatus=nf90_open(trim(fname), NF90_NOWRITE, ncid)
nstatus=nf90_inq_varid(ncid, "LAT", latid)
nstatus=nf90_inq_varid(ncid, "LON", lonid)
nstatus=nf90_inq_varid(ncid, "LWMASK", mid)
nstatus=nf90_get_var(ncid,latid,lat)
nstatus=nf90_get_var(ncid,lonid,lon)
nstatus=nf90_get_var(ncid,mid,lmask)
nstatus=nf90_close(ncid)

end subroutine read_metgrid

subroutine read_modislai(lai,fname)

use netcdf

integer :: i,j
integer, parameter :: nx=2400, ny=2400
real, parameter :: sfact=0.1
integer :: ncid, laid, nstatus
real :: lai(nx,ny)
character*120 :: fname

nstatus=nf90_open(trim(fname), NF90_NOWRITE, ncid)
nstatus=nf90_inq_varid(ncid, "Lai_500m", laid)
nstatus=nf90_get_var(ncid,laid,lai)
nstatus=nf90_close(ncid)

do i=1,nx
 do j=1,ny
  lai(i,j)=sfact*lai(i,j)
  if((lai(i,j).ge.-0.2).or.(lai(i,j).lt.-0.6))then
   lai(i,j)=max(lai(i,j),0.)
  else
   lai(i,j)=-999.
  endif
 enddo
enddo

end subroutine read_modislai

subroutine remap(nx,ny,lat,lon,modislai1,modislai2,nxg,nyg,latg, & 
                 long,lmask,grdspc,lai_remap,nfiles,wt)

integer, intent(in) :: nx, ny
real, intent(in) :: lat(nx,ny),lon(nx,ny), modislai1(nx,ny), modislai2(nx,ny)

integer, intent(in) :: nxg, nyg
real, intent(in) :: latg(nxg,nyg), long(nxg,nyg)
real, intent(inout) :: lai_remap(nxg,nyg)
integer, intent(in) :: lmask(nxg,nyg)

real, intent(in) :: wt, grdspc

integer :: nfiles, i, j, imod, jmod, di
integer :: imax,imin,jmax,jmin
real :: dx, dy, deltax, deltay, pi, errx, erry
real :: width
real :: sum1, sum2, ncount1, ncount2
real :: lai_nn

! set bin width to grid 1/2 spacing of target grid
width = 0.5*grdspc

pi=4.*atan(1.)

! meridional grid spacing 
dy=(maxval(lat)-minval(lat))/float(ny-1)
! indicial width corresponding to bin width
di=nint(width/dy)

do i=1,nxg
 do j=1,nyg

 if (lmask(i,j)==1) then

! j index of nearest MODIS point to WRF grid point
  deltay=lat(1,1)-latg(i,j)
  jmod=nint(deltay/dy)

  if(jmod.ge.1.and.jmod.le.ny)then

!  i index of nearest MODIS point to WRF grid point
   dx=dy/cos(lat(1,jmod)*pi/180.)
   deltax=long(i,j)-lon(1,jmod)
   imod=nint(deltax/dx)

   if(imod.ge.1.and.imod.le.nx)then

     imax=min(imod+di,nx)
     imin=max(imod-di,1)
     jmax=min(jmod+di,ny)
     jmin=max(jmod-di,1)

!    nearest neighbor:
     lai_nn=modislai1(imod,jmod)

!    if nearest neigbhor value is < 0, then use
!    default value from WRF background

     if ( lai_nn >= 0. ) then 

!    bin average:
     if (nfiles==1) then

     lai_remap(i,j)=sum(modislai1(imin:imax,jmin:jmax), & 
                      modislai1(imin:imax,jmin:jmax).ge.0.) / &
                      count(modislai1(imin:imax,jmin:jmax).ge.0.)

     else

     lai_remap(i,j)=wt*sum(modislai1(imin:imax,jmin:jmax), &
                       modislai1(imin:imax,jmin:jmax).ge.0.) / &
                       count(modislai1(imin:imax,jmin:jmax).ge.0.) + &
               (1.-wt)*sum(modislai2(imin:imax,jmin:jmax), &
                       modislai2(imin:imax,jmin:jmax).ge.0.) / &
                       count(modislai2(imin:imax,jmin:jmax).ge.0.)

     endif

     else

      lai_remap(i,j)=-999.

     endif

   endif
  endif

  endif

 enddo
enddo

end subroutine remap

subroutine write_modislai(nx,ny,lai,lmask,lat,lon,fname)

use netcdf

integer :: i,j,nx,ny
real, parameter :: sfact=0.1
integer, parameter :: nlev=25
integer :: dimids(2)
integer :: ncid, varid, nstatus, x_dimid, y_dimid
integer :: lmask(nx,ny)
real :: lai(nx,ny), lat(nx,ny), lon(nx,ny)
real :: var_out(nx,ny,1,nlev), bglai(nx,ny,1,nlev)
character*120 :: fname

nstatus=nf90_open(trim(fname), NF90_WRITE, ncid) 
nstatus=nf90_inq_varid(ncid, "LAI", varid)
nstatus=nf90_get_var(ncid, varid, bglai)

! update only those points for which the remapped LAI is defined

do k=1,nlev
 do i=1,nx
  do j=1,ny
    if ((lai(i,j)>=0.).and.(lmask(i,j)==1)) then
     var_out(i,j,1,k)=lai(i,j)
    else
     var_out(i,j,1,k)=bglai(i,j,1,k)
    endif
  enddo
 enddo
enddo

nstatus=nf90_put_var(ncid, varid, var_out)
nstatus=nf90_close(ncid) 

end subroutine write_modislai
